%%%-------------------------------------------------------------------
%%% @author Piotr Duleba
%%% @copyright (C) 2020 ACK CYFRONET AGH
%%% This software is released under the MIT license
%%% cited in 'LICENSE.txt'.
%%% @end
%%%-------------------------------------------------------------------
%%% @doc
%%% This file provides functions for making rpc calls to Onepanel Nodes for use during tests.
%%% @end
%%%-------------------------------------------------------------------
-module(panel_test_rpc).
-author("Piotr Duleba").

-include_lib("ctool/include/errors.hrl").
-include_lib("ctool/include/test/test_utils.hrl").

%% API
-export([
    call/4,

    get_op_worker_nodes/1,
    get_cert_chain_ders/1,
    get_cluster_id/1,
    create_invite_token/1,
    is_emergency_passphrase_set/1,
    set_emergency_passphrase/2,
    unset_emergency_passphrase/1,
    verify_emergency_passphrase/2,
    list_onepanel_deployment/1,
    is_onepanel_initialized/1
]).

-define(CALL(NodeSelector, Args), test_rpc:call(onepanel, NodeSelector, test_rpc_api, ?FUNCTION_NAME, Args)).


%%%===================================================================
%%% API functions
%%%===================================================================


-spec call(oct_background:node_selector(), module(), atom(), [term()]) -> term() | no_return().
call(NodeSelector, Module, Function, Args) ->
    test_rpc:call(onepanel, NodeSelector, Module, Function, Args).


-spec get_op_worker_nodes(oct_background:node_selector()) -> [node()].
get_op_worker_nodes(NodeSelector) ->
    ?CALL(NodeSelector, []).


-spec get_cert_chain_ders(oct_background:node_selector()) -> [public_key:der_encoded()].
get_cert_chain_ders(NodeSelector) ->
    ?CALL(NodeSelector, []).


-spec get_cluster_id(oct_background:node_selector()) -> binary().
get_cluster_id(NodeSelector) ->
    ?CALL(NodeSelector, []).


-spec create_invite_token(oct_background:node_selector()) -> binary().
create_invite_token(NodeSelector) ->
    {ok, Token} = ?assertMatch({ok, _}, ?CALL(NodeSelector, [])),
    Token.


-spec is_emergency_passphrase_set(oct_background:node_selector()) -> boolean().
is_emergency_passphrase_set(NodeSelector) ->
    ?CALL(NodeSelector, []).


-spec set_emergency_passphrase(oct_background:node_selector(), binary()) -> ok | ?ERROR_BAD_VALUE_PASSWORD.
set_emergency_passphrase(Nodeselector, NewPassphrase) ->
    ?CALL(Nodeselector, [NewPassphrase]).


-spec unset_emergency_passphrase(oct_background:node_selector()) -> ok | no_return().
unset_emergency_passphrase(Nodeselector) ->
    ?CALL(Nodeselector, []).


-spec verify_emergency_passphrase(oct_background:node_selector(), binary()) -> boolean().
verify_emergency_passphrase(NodeSelector, Passphrase) ->
    ?CALL(NodeSelector, [Passphrase]).


-spec list_onepanel_deployment(oct_background:node_selector()) -> list().
list_onepanel_deployment(NodeSelector) ->
    ?CALL(NodeSelector, []).


-spec is_onepanel_initialized(oct_background:node_selector()) -> boolean().
is_onepanel_initialized(NodeSelector) ->
    ?CALL(NodeSelector, []).
